use std::io::Read;
use std::str::FromStr;

use ethereum_types::H256;
use keccak_hash::keccak;
use mpt_trie::{
    nibbles::Nibbles,
    partial_trie::{HashedPartialTrie, PartialTrie},
    trie_ops::ValOrHash,
};

extern crate ethereum_types;
extern crate keccak_hash;
extern crate mpt_trie;

/// hash
pub fn hash(bytes: &[u8]) -> H256 {
    H256::from(keccak(bytes).0)
}

fn main() {
    let mut buffer = String::new();
    std::io::stdin().read_to_string(&mut buffer).unwrap();

    let des = &mut serde_json::Deserializer::from_str(&buffer);
    let mut input: HashedPartialTrie = serde_path_to_error::deserialize(des).unwrap();

    println!("initial hash: {}", input.hash());

    let insert_key =
        Nibbles::from_str("0xe7c11d5270a96d8ff353e9c32fc53375eaeedd3efe2be73798d560e1d8c1f299")
            .unwrap();
    let insert_value: Vec<u8> = vec![137, 14, 108, 227, 242, 56, 61, 189, 73, 108];

    println!("insert_key: {:?}", insert_key);
    println!("insert_value: {:?}", insert_value);

    println!("do insert");

    input
        .insert(
            Nibbles::from_h256_be(hash(&insert_key.bytes_be())),
            ValOrHash::Val(insert_value),
        )
        .expect("Insert failed");

    println!("hash after insert: {}", input.hash());

    let delete_key =
        Nibbles::from_str("0xcec0d88d45c06fe2864991abe6cf6a1fbbd064d8166c0c24cd36eeb1016f9282")
            .unwrap();

    println!("delete_key: {:?}", delete_key);

    let initial_del_value = input.get(Nibbles::from_h256_be(hash(&delete_key.bytes_be())));

    println!("value in slot before delete: {:?}", initial_del_value);

    input
        .delete(Nibbles::from_h256_be(hash(&delete_key.bytes_be())))
        .expect("Delete failed");

    println!("hash after delete: {}", input.hash());
}
